const {parseString} = require('xml2js')
const {compose} = require('@simonjayhawkins/urban-chainsaw')
const COLUMNS = require('./columns')

module.exports = {
  parseXML: parseXML,
  parseJSON: parseJSON,
  transformJSON: transformJSON
}

function parseXML (xml, callback) {
  parseString(xml,
    {explicitRoot: false},
    (err, result) => (err) ? callback(err) : callback(null, parseGetMatchingProductForIdResponse(result))
  )
}

function parseJSON (json, callback) {
  const timestamp = json.ResponseHeaderMetadata['x-mws-timestamp']

  if (!timestamp) return callback(new Error('Invalid Timestamp'))

  parseXML(json.Response, (err, data) => {
    if (err) return callback(err)

    data.forEach(value => {
      value.timestamp = timestamp
    })

    callback(null, data)
  })
}

function parseGetMatchingProductForIdResponse (response) {
  const isObject = obj => obj && typeof obj === 'object'
  const isEmptyObject = obj => isObject(obj) && Object.keys(obj).length === 0

  const removeEmptyObjects = obj => {
    for (let key of Object.keys(obj)) {
      if (isEmptyObject(obj[key])) obj[key] = undefined
    }
  }

  const undef = new Proxy({}, {
    get: (target, name) => undef
  })

  const handler = {
    get: (target, name) => {
      if (name in target && target[name]) return (isObject(target[name])) ? new Proxy(target[name], handler) : target[name]
      else return undef
    }
  }

  const p = new Proxy(response, handler)

  const array = []

  const RequestId = p.ResponseMetadata[0].RequestId[0]

  p.GetMatchingProductForIdResult
    .forEach(GetMatchingProductForIdResult => {
      const Id = GetMatchingProductForIdResult.$.Id
      const status = GetMatchingProductForIdResult.$.status
      if (status !== 'Success') {
        array.push({
          Id: Id,
          status: status,
          ErrorMessage: GetMatchingProductForIdResult.Error[0].Message[0],
          RequestId: RequestId
        })
      } else {
        const Products = GetMatchingProductForIdResult.Products[0].Product
        if (Array.isArray(Products)) {
          Products.forEach(Product => {
            array.push({
              Id: Id,
              status: status,
              ASIN: Product.Identifiers[0].MarketplaceASIN[0].ASIN[0],
              Title: Product.AttributeSets[0]['ns2:ItemAttributes'][0]['ns2:Title'][0],
              ListPrice: Product.AttributeSets[0]['ns2:ItemAttributes'][0]['ns2:ListPrice'][0]['ns2:Amount'][0],
              CurrencyCode: Product.AttributeSets[0]['ns2:ItemAttributes'][0]['ns2:ListPrice'][0]['ns2:CurrencyCode'][0],
              ProductCategoryId: Product.SalesRankings[0].SalesRank[0].ProductCategoryId[0],
              Rank: Product.SalesRankings[0].SalesRank[0].Rank[0],
              RequestId: RequestId
            })
          })
        } else {
          array.push({
            Id: Id,
            status: status,
            RequestId: RequestId
          })
        }
      }
    })

  array.forEach(obj => removeEmptyObjects(obj))
  return array
}

function transformJSON (json, header, callback) {
  parseJSON(json, (err, data) => {
    if (err) return callback(err)

    data.forEach(value => {
      value.timestamp = value.timestamp.replace(/\.\d{3}Z/, 'Z')
    })

    const output = compose(data, {
      header: header,
      columns: COLUMNS
    })

    callback(null, output)
  })
}
