/* global it, describe */

const {assert} = require('chai')
const sinon = require('sinon')
const fs = require('fs')

const DataService = require('../src/data-service-local-fs')

const dataService = new DataService({
  eventSourceFile: './test/fixtures/matchresults.log.sample',
  snapshotFile: '/dev/null'
})

const mockResponse = JSON.parse(fs.readFileSync('test/fixtures/1489518b-8c37-44f7-ae53-EXAMPLEd89db0.json', 'utf8'))

describe('data-service-local-fs module', function () {
  describe('init method', function () {
    it('should update snapshot from log on initialisation', function (done) {
      const stub = sinon.stub(dataService, 'updateSnapshot').callsArg(0)
      dataService.init(function () {
        assert.isTrue(stub.called)
        stub.restore()
        done()
      })
    })
  })

  describe('save method', function () {
    it('should call the processResponse methods', function (done) {
      const processResponseStub = sinon.stub(dataService, 'processResponse').callsArg(1)
      dataService.save(mockResponse, function (err) {
        assert.isTrue(processResponseStub.called)
        processResponseStub.restore()
        done(err)
      })
    })

    it('should return an error if processResponse returns an error', function (done) {
      const processResponseStub = sinon.stub(dataService, 'processResponse').callsArgWith(1, 'an error occurred during processing')
      dataService.save(mockResponse, function (err, results) {
        processResponseStub.restore()
        done(!err)
      })
    })
  })

  describe('processResponse method', function () {
    it('should be tested')
  })
})
